@@ -1,5 +1,4 @@
 from .claims import Claims
-from .claims_builder import ClaimsBuilder, claims
 from .cose import COSE
 from .cose_key import COSEKey
 from .cwt import (
@@ -11,11 +10,11 @@ from .cwt import (
     encode_and_sign,
     set_private_claim_names,
 )
+from .encrypted_cose_key import EncryptedCOSEKey
 from .exceptions import CWTError, DecodeError, EncodeError, VerifyError
-from .key_builder import KeyBuilder, cose_key
 from .recipient import Recipient
 
-__version__ = "0.7.1"
+__version__ = "0.8.1"
 __title__ = "cwt"
 __description__ = "A Python implementation of CWT/COSE"
 __url__ = "https://python-cwt.readthedocs.io"
@@ -34,13 +33,10 @@ __all__ = [
     "set_private_claim_names",
     "CWT",
     "COSE",
-    "Claims",
-    "ClaimsBuilder",
     "COSEKey",
-    "KeyBuilder",
-    "cose_key",
+    "EncryptedCOSEKey",
+    "Claims",
     "Recipient",
-    "claims",
     "CWTError",
     "EncodeError",
     "DecodeError",
