@@ -35,6 +35,28 @@ CWT_CLAIM_NAMES = {
     # "intuse": 41,  # integer(1, 2, 3, 4, 5)
 }
 
+# COSE Header Parameters
+COSE_HEADER_PARAMETERS = {
+    "alg": 1,
+    "crit": 2,
+    "cty": 3,
+    "content type": 3,
+    "kid": 4,
+    "iv": 5,
+    "IV": 5,
+    "Partial IV": 6,
+    "counter signature": 7,
+    "CounterSignature0": 9,
+    "kid context": 10,
+    "x5bag": 32,
+    "x5c": 33,
+    "x5chain": 33,
+    "x5t": 34,
+    "x5u": 35,
+    "CUPHNonce": 256,
+    "CUPHOwnerPubKey": 257,
+}
+
 # COSE key types
 COSE_KEY_TYPES = {
     "OKP": 1,  # OCtet Key Pair
@@ -122,6 +144,38 @@ COSE_ALGORITHMS_CEK = {
     # etc.
 }
 
+COSE_KEY_LEN = {
+    -5: 256,  # AES Key Wrap w/ 256-bit key
+    -4: 192,  # AES Key Wrap w/ 192-bit key
+    -3: 128,  # AES Key Wrap w/ 128-bit key
+    1: 128,  # AES-GCM mode w/ 128-bit key, 128-bit tag
+    2: 192,  # AES-GCM mode w/ 192-bit key, 128-bit tag
+    3: 256,  # AES-GCM mode w/ 256-bit key, 128-bit tag
+    10: 128,  # AES-CCM mode 128-bit key, 64-bit tag, 13-byte nonce
+    11: 256,  # AES-CCM mode 256-bit key, 64-bit tag, 13-byte nonce
+    12: 128,  # AES-CCM mode 128-bit key, 64-bit tag, 7-byte nonce
+    13: 256,  # AES-CCM mode 256-bit key, 64-bit tag, 7-byte nonce
+    24: 256,  # ChaCha20/Poly1305 w/ 256-bit key, 128-bit tag
+    30: 128,  # AES-CCM mode 128-bit key, 128-bit tag, 13-byte nonce
+    31: 256,  # AES-CCM mode 256-bit key, 128-bit tag, 13-byte nonce
+    32: 128,  # AES-CCM mode 128-bit key, 128-bit tag, 7-byte nonce
+    33: 256,  # AES-CCM mode 256-bit key, 128-bit tag, 7-byte nonce
+}
+
+COSE_ALGORITHMS_CKDM = {
+    "direct+HKDF-SHA-512": -11,  # Shared secret w/ HKDF and SHA-512
+    "direct+HKDF-SHA-256": -10,  # Shared secret w/ HKDF and SHA-256
+    "direct": -6,  # direct
+    # etc.
+}
+
+COSE_ALGORITHMS_KEY_WRAP = {
+    "A256KW": -5,  # AES Key Wrap w/ 256-bit key
+    "A192KW": -4,  # AES Key Wrap w/ 192-bit key
+    "A128KW": -3,  # AES Key Wrap w/ 128-bit key
+    # etc.
+}
+
 # COSE Algorithms for MAC.
 COSE_ALGORITHMS_MAC = {
     "HMAC 256/64": 4,  # HMAC w/ SHA-256 truncated to 64 bits
@@ -201,6 +255,9 @@ JWK_ELLIPTIC_CURVES = {
     "secp256k1": 8,
 }
 
+# COSE Algorithms for recipients.
+COSE_ALGORITHMS_RECIPIENT = dict(COSE_ALGORITHMS_CKDM, **COSE_ALGORITHMS_KEY_WRAP)
+
 # COSE Algorithms for Symmetric Keys.
 COSE_ALGORITHMS_SYMMETRIC = dict(COSE_ALGORITHMS_MAC, **COSE_ALGORITHMS_CEK)
 
