@@ -3,48 +3,47 @@ from typing import Any, Dict, List, Optional, Union
 from cbor2 import CBORTag
 
 from .cbor_processor import CBORProcessor
-from .cose_key import COSEKey
-from .recipient import Recipient, RecipientsBuilder
+from .const import COSE_ALGORITHMS_RECIPIENT
+from .cose_key_interface import COSEKeyInterface
+from .recipient_interface import RecipientInterface
+from .recipients import Recipients
+from .utils import to_cose_header
 
 
 class COSE(CBORProcessor):
     """
     A COSE (CBOR Object Signing and Encryption) Implementaion built on top of
     `cbor2 <https://cbor2.readthedocs.io/en/stable/>`_.
-
-    ``cwt.cose_key`` is a global object of this class initialized with default settings.
     """
 
-    def __init__(self, options: Optional[Dict[str, Any]] = None):
+    def __init__(
+        self, alg_auto_inclusion: int = False, kid_auto_inclusion: int = False
+    ):
         """
         Constructor.
 
         Args:
-            options (Optional[Dict[str, Any]]): Options for the initial configuration
-                of COSE. At this time, ``kid_auto_inclusion`` (default value: ``True``)
-                and ``alg_auto_inclusion`` (default value: ``True``) are supported.
+            alg_auto_inclusion(bool): The indicator whether ``alg`` parameter is included
+                in a proper header bucket automatically or not.
+            kid_auto_inclusion(bool): The indicator whether ``kid`` parameter is included
+                in a proper header bucket automatically or not.
         """
-        self._recipients_builder = RecipientsBuilder()
-        self._kid_auto_inclusion = True
-        self._alg_auto_inclusion = True
-        if not options:
-            return
-        if "kid_auto_inclusion" in options:
-            if not isinstance(options["kid_auto_inclusion"], bool):
-                raise ValueError("kid_auto_inclusion should be bool.")
-            self._kid_auto_inclusion = options["kid_auto_inclusion"]
-        if "alg_auto_inclusion" in options:
-            if not isinstance(options["alg_auto_inclusion"], bool):
-                raise ValueError("alg_auto_inclusion should be bool.")
-            self._alg_auto_inclusion = options["alg_auto_inclusion"]
+        if not isinstance(alg_auto_inclusion, bool):
+            raise ValueError("alg_auto_inclusion should be bool.")
+        self._alg_auto_inclusion = alg_auto_inclusion
+
+        if not isinstance(kid_auto_inclusion, bool):
+            raise ValueError("kid_auto_inclusion should be bool.")
+        self._kid_auto_inclusion = kid_auto_inclusion
 
     def encode_and_mac(
         self,
         payload: bytes,
-        key: COSEKey,
-        protected: Optional[Union[Dict[int, Any], bytes]] = None,
-        unprotected: Optional[Dict[int, Any]] = None,
-        recipients: Optional[List[Recipient]] = None,
+        key: COSEKeyInterface,
+        protected: Optional[Union[dict, bytes]] = None,
+        unprotected: Optional[dict] = None,
+        recipients: Optional[List[RecipientInterface]] = None,
+        external_aad: bytes = b"",
         out: str = "",
     ) -> Union[bytes, CBORTag]:
         """
@@ -52,11 +51,12 @@ class COSE(CBORProcessor):
 
         Args:
             payload (bytes): A content to be MACed.
-            key (COSEKey): A COSE key as a MAC Authentication key.
-            protected (Union[Dict[int, Any], bytes]): Parameters that are to be cryptographically
+            key (COSEKeyInterface): A COSE key as a MAC Authentication key.
+            protected (Optional[Union[dict, bytes]]): Parameters that are to be cryptographically
                 protected.
-            unprotected (Dict[int, Any]): Parameters that are not cryptographically protected.
-            recipients (Optional[List[Recipient]]): A list of recipient information structures.
+            unprotected (Optional[dict]): Parameters that are not cryptographically protected.
+            recipients (Optional[List[RecipientInterface]]): A list of recipient information structures.
+            external_aad(bytes): External additional authenticated data supplied by application.
             out(str): An output format. Only ``"cbor2/CBORTag"`` can be used. If ``"cbor2/CBORTag"``
                 is specified. This function will return encoded data as
                 `cbor2 <https://cbor2.readthedocs.io/en/stable/>`_'s ``CBORTag`` object.
@@ -67,49 +67,51 @@ class COSE(CBORProcessor):
             ValueError: Invalid arguments.
             EncodeError: Failed to encode data.
         """
-        protected = {} if protected is None else protected
-        unprotected = {} if unprotected is None else unprotected
+        p: Union[Dict[int, Any], bytes] = (
+            to_cose_header(protected) if not isinstance(protected, bytes) else protected
+        )
+        u = to_cose_header(unprotected)
 
         ctx = "MAC0" if not recipients else "MAC"
         b_protected = b""
 
         # MAC0
         if not recipients:
-            if isinstance(protected, bytes):
-                b_protected = protected
+            if isinstance(p, bytes):
+                b_protected = p
             else:
                 if self._alg_auto_inclusion:
-                    protected[1] = key.alg
+                    p[1] = key.alg
                 if self._kid_auto_inclusion and key.kid:
-                    unprotected[4] = key.kid
-                b_protected = self._dumps(protected)
-            mac_structure = [ctx, b_protected, b"", payload]
+                    u[4] = key.kid
+                b_protected = self._dumps(p)
+            mac_structure = [ctx, b_protected, external_aad, payload]
             tag = key.sign(self._dumps(mac_structure))
-            res = CBORTag(17, [b_protected, unprotected, payload, tag])
+            res = CBORTag(17, [b_protected, u, payload, tag])
             return res if out == "cbor2/CBORTag" else self._dumps(res)
 
         # MAC
         recs = []
         for rec in recipients:
             recs.append(rec.to_list())
-        if recipients[0].alg == -6:
-            if not isinstance(protected, bytes):
+        if recipients[0].alg in COSE_ALGORITHMS_RECIPIENT.values():
+            if not isinstance(p, bytes):
                 if self._alg_auto_inclusion:
-                    protected[1] = key.alg
+                    p[1] = key.alg
                 if self._kid_auto_inclusion and key.kid:
-                    unprotected[4] = key.kid
+                    u[4] = key.kid
         else:
             raise NotImplementedError(
                 "Algorithms other than direct are not supported for recipients."
             )
 
-        if isinstance(protected, bytes):
-            b_protected = protected
+        if isinstance(p, bytes):
+            b_protected = p
         else:
-            b_protected = self._dumps(protected) if protected else b""
-        mac_structure = [ctx, b_protected, b"", payload]
+            b_protected = self._dumps(p) if p else b""
+        mac_structure = [ctx, b_protected, external_aad, payload]
         tag = key.sign(self._dumps(mac_structure))
-        cose_mac: List[Any] = [b_protected, unprotected, payload, tag]
+        cose_mac: List[Any] = [b_protected, u, payload, tag]
         cose_mac.append(recs)
         res = CBORTag(97, cose_mac)
         return res if out == "cbor2/CBORTag" else self._dumps(res)
@@ -117,9 +119,10 @@ class COSE(CBORProcessor):
     def encode_and_sign(
         self,
         payload: bytes,
-        key: Union[COSEKey, List[COSEKey]],
-        protected: Optional[Union[Dict[int, Any], bytes]] = None,
-        unprotected: Optional[Dict[int, Any]] = None,
+        key: Union[COSEKeyInterface, List[COSEKeyInterface]],
+        protected: Optional[Union[dict, bytes]] = None,
+        unprotected: Optional[dict] = None,
+        external_aad: bytes = b"",
         out: str = "",
     ) -> Union[bytes, CBORTag]:
         """
@@ -127,11 +130,11 @@ class COSE(CBORProcessor):
 
         Args:
             payload (bytes): A content to be signed.
-            key (Union[COSEKey, List[COSEKey]]): One or more COSE keys as signing keys.
-            protected (Union[Dict[int, Any], bytes]): Parameters that are to be cryptographically
-                protected.
-            unprotected (Dict[int, Any]): Parameters that are not cryptographically
+            key (Union[COSEKeyInterface, List[COSEKeyInterface]]): One or more COSE keys as signing keys.
+            protected (Optional[Union[dict, bytes]]): Parameters that are to be cryptographically
                 protected.
+            unprotected (Optional[dict]): Parameters that are not cryptographically protected.
+            external_aad(bytes): External additional authenticated data supplied by application.
             out(str): An output format. Only ``"cbor2/CBORTag"`` can be used. If ``"cbor2/CBORTag"``
                 is specified. This function will return encoded data as
                 `cbor2 <https://cbor2.readthedocs.io/en/stable/>`_'s ``CBORTag`` object.
@@ -142,27 +145,29 @@ class COSE(CBORProcessor):
             ValueError: Invalid arguments.
             EncodeError: Failed to encode data.
         """
-        protected = {} if protected is None else protected
-        unprotected = {} if unprotected is None else unprotected
+        p: Union[Dict[int, Any], bytes] = (
+            to_cose_header(protected) if not isinstance(protected, bytes) else protected
+        )
+        u = to_cose_header(unprotected)
 
-        ctx = "Signature" if not isinstance(key, COSEKey) else "Signature1"
-        if isinstance(key, COSEKey) and isinstance(protected, dict):
+        ctx = "Signature" if not isinstance(key, COSEKeyInterface) else "Signature1"
+        if isinstance(key, COSEKeyInterface) and isinstance(p, dict):
             if self._alg_auto_inclusion:
-                protected[1] = key.alg
+                p[1] = key.alg
             if self._kid_auto_inclusion and key.kid:
-                unprotected[4] = key.kid
+                u[4] = key.kid
 
         b_protected = b""
-        if isinstance(protected, bytes):
-            b_protected = protected
+        if isinstance(p, bytes):
+            b_protected = p
         else:
-            b_protected = self._dumps(protected) if protected else b""
+            b_protected = self._dumps(p) if p else b""
 
         # Signature1
-        if isinstance(key, COSEKey):
-            sig_structure = [ctx, b_protected, b"", payload]
+        if isinstance(key, COSEKeyInterface):
+            sig_structure = [ctx, b_protected, external_aad, payload]
             sig = key.sign(self._dumps(sig_structure))
-            res = CBORTag(18, [b_protected, unprotected, payload, sig])
+            res = CBORTag(18, [b_protected, u, payload, sig])
             return res if out == "cbor2/CBORTag" else self._dumps(res)
 
         # Signature
@@ -170,20 +175,21 @@ class COSE(CBORProcessor):
         for k in key:
             p_header = self._dumps({1: k.alg})
             u_header = {4: k.kid} if k.kid else {}
-            sig_structure = [ctx, b_protected, p_header, b"", payload]
+            sig_structure = [ctx, b_protected, p_header, external_aad, payload]
             sig = k.sign(self._dumps(sig_structure))
             sigs.append([p_header, u_header, sig])
-        res = CBORTag(98, [b_protected, unprotected, payload, sigs])
+        res = CBORTag(98, [b_protected, u, payload, sigs])
         return res if out == "cbor2/CBORTag" else self._dumps(res)
 
     def encode_and_encrypt(
         self,
         payload: bytes,
-        key: COSEKey,
-        protected: Optional[Union[Dict[int, Any], bytes]] = None,
-        unprotected: Optional[Dict[int, Any]] = None,
+        key: COSEKeyInterface,
+        protected: Optional[Union[dict, bytes]] = None,
+        unprotected: Optional[dict] = None,
         nonce: bytes = b"",
-        recipients: Optional[List[Recipient]] = None,
+        recipients: Optional[List[RecipientInterface]] = None,
+        external_aad: bytes = b"",
         out: str = "",
     ) -> bytes:
         """
@@ -191,13 +197,13 @@ class COSE(CBORProcessor):
 
         Args:
             payload (bytes): A content to be encrypted.
-            key (COSEKey): A COSE key as an encryption key.
-            protected (Union[Dict[int, Any], bytes]): Parameters that are to be cryptographically
-                protected.
-            unprotected (Dict[int, Any]): Parameters that are not cryptographically
+            key (COSEKeyInterface): A COSE key as an encryption key.
+            protected (Optional[Union[dict, bytes]]): Parameters that are to be cryptographically
                 protected.
+            unprotected (Optional[dict]): Parameters that are not cryptographically protected.
             nonce (bytes): A nonce for encryption.
-            recipients (Optional[List[Recipient]]): A list of recipient information structures.
+            recipients (Optional[List[RecipientInterface]]): A list of recipient information structures.
+            external_aad(bytes): External additional authenticated data supplied by application.
             out(str): An output format. Only ``"cbor2/CBORTag"`` can be used. If ``"cbor2/CBORTag"``
                 is specified. This function will return encoded data as
                 `cbor2 <https://cbor2.readthedocs.io/en/stable/>`_'s ``CBORTag`` object.
@@ -208,8 +214,10 @@ class COSE(CBORProcessor):
             ValueError: Invalid arguments.
             EncodeError: Failed to encode data.
         """
-        protected = {} if protected is None else protected
-        unprotected = {} if unprotected is None else unprotected
+        p: Union[Dict[int, Any], bytes] = (
+            to_cose_header(protected) if not isinstance(protected, bytes) else protected
+        )
+        u = to_cose_header(unprotected)
 
         ctx = "Encrypt0" if not recipients else "Encrypt"
 
@@ -223,49 +231,53 @@ class COSE(CBORProcessor):
 
         # Encrypt0
         if not recipients:
-            if isinstance(protected, bytes):
-                b_protected = protected
+            if isinstance(p, bytes):
+                b_protected = p
             else:
                 if self._alg_auto_inclusion:
-                    protected[1] = key.alg
-                b_protected = self._dumps(protected) if protected else b""
+                    p[1] = key.alg
+                b_protected = self._dumps(p) if p else b""
             if self._kid_auto_inclusion and key.kid:
-                unprotected[4] = key.kid
-            unprotected[5] = nonce
-            enc_structure = [ctx, b_protected, b""]
+                u[4] = key.kid
+            u[5] = nonce
+            enc_structure = [ctx, b_protected, external_aad]
             aad = self._dumps(enc_structure)
             ciphertext = key.encrypt(payload, nonce, aad)
-            res = CBORTag(16, [b_protected, unprotected, ciphertext])
+            res = CBORTag(16, [b_protected, u, ciphertext])
             return res if out == "cbor2/CBORTag" else self._dumps(res)
 
         # Encrypt
         recs = []
         for rec in recipients:
             recs.append(rec.to_list())
-        if recipients[0].alg == -6:
-            if not isinstance(protected, bytes) and self._alg_auto_inclusion:
-                protected[1] = key.alg
+        if recipients[0].alg in COSE_ALGORITHMS_RECIPIENT.values():
+            if not isinstance(p, bytes) and self._alg_auto_inclusion:
+                p[1] = key.alg
             if self._kid_auto_inclusion and key.kid:
-                unprotected[4] = key.kid
-            unprotected[5] = nonce
+                u[4] = key.kid
+            u[5] = nonce
         else:
             raise NotImplementedError(
                 "Algorithms other than direct are not supported for recipients."
             )
-        if isinstance(protected, bytes):
-            b_protected = protected
+        if isinstance(p, bytes):
+            b_protected = p
         else:
-            b_protected = self._dumps(protected) if protected else b""
-        enc_structure = [ctx, b_protected, b""]
+            b_protected = self._dumps(p) if p else b""
+        enc_structure = [ctx, b_protected, external_aad]
         aad = self._dumps(enc_structure)
         ciphertext = key.encrypt(payload, nonce, aad)
-        cose_enc: List[Any] = [b_protected, unprotected, ciphertext]
+        cose_enc: List[Any] = [b_protected, u, ciphertext]
         cose_enc.append(recs)
         res = CBORTag(96, cose_enc)
         return res if out == "cbor2/CBORTag" else self._dumps(res)
 
     def decode(
-        self, data: Union[bytes, CBORTag], key: Union[COSEKey, List[COSEKey]]
+        self,
+        data: Union[bytes, CBORTag],
+        key: Optional[Union[COSEKeyInterface, List[COSEKeyInterface]]] = None,
+        materials: Optional[List[dict]] = None,
+        external_aad: bytes = b"",
     ) -> bytes:
         """
         Verifies and decodes COSE data.
@@ -273,7 +285,9 @@ class COSE(CBORProcessor):
         Args:
             data (Union[bytes, CBORTag]): A byte string or cbor2.CBORTag of an
                 encoded data.
-            key (COSEKey): A COSE key to verify and decrypt the encoded data.
+            key (Optional[Union[COSEKeyInterface, List[COSEKeyInterface]]]): A COSE key to verify and decrypt the encoded data.
+            materials (Optional[List[dict]]): A list of key materials to be used to derive an encryption key.
+            external_aad(bytes): External additional authenticated data supplied by application.
         Returns:
             bytes: A byte string of decoded payload.
         Raises:
@@ -281,12 +295,16 @@ class COSE(CBORProcessor):
             DecodeError: Failed to decode data.
             VerifyError: Failed to verify data.
         """
+        if key is None and materials is None:
+            raise ValueError("Either key or materials should be specified.")
         if isinstance(data, bytes):
             data = self._loads(data)
         if not isinstance(data, CBORTag):
             raise ValueError("Invalid COSE format.")
 
-        keys: List[COSEKey] = key if isinstance(key, list) else [key]
+        keys: List[COSEKeyInterface] = []
+        if key:
+            keys = key if isinstance(key, list) else [key]
 
         # Encrypt0
         if data.tag == 16:
@@ -294,7 +312,7 @@ class COSE(CBORProcessor):
             if not isinstance(data.value, list) or len(data.value) != 3:
                 raise ValueError("Invalid Encrypt0 format.")
 
-            aad = self._dumps(["Encrypt0", data.value[0], b""])
+            aad = self._dumps(["Encrypt0", data.value[0], external_aad])
             unprotected = data.value[1]
             if not isinstance(unprotected, dict):
                 raise ValueError("unprotected header should be dict.")
@@ -306,17 +324,30 @@ class COSE(CBORProcessor):
 
         # Encrypt
         if data.tag == 96:
-            keys = self._filter_by_key_ops(keys, 4)
-            if not isinstance(data.value, list) or len(data.value) != 4:
-                raise ValueError("Invalid Encrypt format.")
+            if keys:
+                keys = self._filter_by_key_ops(keys, 4)
+                if not isinstance(data.value, list) or len(data.value) != 4:
+                    raise ValueError("Invalid Encrypt format.")
 
-            aad = self._dumps(["Encrypt", data.value[0], b""])
+            aad = self._dumps(["Encrypt", data.value[0], external_aad])
+            alg_hint = 0
+            if data.value[0]:
+                protected = self._loads(data.value[0])
+                alg_hint = (
+                    protected[1]
+                    if isinstance(protected, dict) and 1 in protected
+                    else 0
+                )
             unprotected = data.value[1]
             if not isinstance(unprotected, dict):
                 raise ValueError("unprotected header should be dict.")
             nonce = unprotected.get(5, None)
-            recipients = self._recipients_builder.from_list(data.value[3])
-            enc_key = recipients.derive_key(keys)
+            recipients = Recipients.from_list(data.value[3])
+            enc_key = (
+                recipients.derive_key(keys=keys, alg_hint=alg_hint)
+                if key is not None
+                else recipients.derive_key(materials=materials, alg_hint=alg_hint)
+            )
             return enc_key.decrypt(data.value[2], nonce, aad)
 
         # MAC0
@@ -325,7 +356,7 @@ class COSE(CBORProcessor):
             if not isinstance(data.value, list) or len(data.value) != 4:
                 raise ValueError("Invalid MAC0 format.")
 
-            msg = self._dumps(["MAC0", data.value[0], b"", data.value[2]])
+            msg = self._dumps(["MAC0", data.value[0], external_aad, data.value[2]])
             k = self._get_key(keys, data.value[1])
             if not k:
                 raise ValueError("key is not specified.")
@@ -337,9 +368,19 @@ class COSE(CBORProcessor):
             keys = self._filter_by_key_ops(keys, 10)
             if not isinstance(data.value, list) or len(data.value) != 5:
                 raise ValueError("Invalid MAC format.")
-            to_be_maced = self._dumps(["MAC", data.value[0], b"", data.value[2]])
-            recipients = self._recipients_builder.from_list(data.value[4])
-            mac_auth_key = recipients.derive_key(keys)
+            to_be_maced = self._dumps(
+                ["MAC", data.value[0], external_aad, data.value[2]]
+            )
+            alg_hint = 0
+            if data.value[0]:
+                protected = self._loads(data.value[0])
+                alg_hint = (
+                    protected[1]
+                    if isinstance(protected, dict) and 1 in protected
+                    else 0
+                )
+            recipients = Recipients.from_list(data.value[4])
+            mac_auth_key = recipients.derive_key(keys=keys, alg_hint=alg_hint)
             mac_auth_key.verify(to_be_maced, data.value[3])
             return data.value[2]
 
@@ -350,7 +391,7 @@ class COSE(CBORProcessor):
                 raise ValueError("Invalid Signature1 format.")
 
             to_be_signed = self._dumps(
-                ["Signature1", data.value[0], b"", data.value[2]]
+                ["Signature1", data.value[0], external_aad, data.value[2]]
             )
             k = self._get_key(keys, data.value[1])
             if not k:
@@ -373,7 +414,7 @@ class COSE(CBORProcessor):
                 if not k:
                     continue
                 to_be_signed = self._dumps(
-                    ["Signature", data.value[0], sig[0], b"", data.value[2]]
+                    ["Signature", data.value[0], sig[0], external_aad, data.value[2]]
                 )
                 k.verify(to_be_signed, sig[2])
                 return data.value[2]
@@ -381,8 +422,8 @@ class COSE(CBORProcessor):
         raise ValueError(f"Unsupported or unknown CBOR tag({data.tag}).")
 
     def _get_key(
-        self, keys: List[COSEKey], unprotected: Dict[int, Any]
-    ) -> Union[COSEKey, None]:
+        self, keys: List[COSEKeyInterface], unprotected: Dict[int, Any]
+    ) -> Union[COSEKeyInterface, None]:
         if len(keys) == 1:
             if 4 in unprotected and keys[0].kid:
                 if unprotected[4] != keys[0].kid:
@@ -395,8 +436,10 @@ class COSE(CBORProcessor):
                 return k
         return None
 
-    def _filter_by_key_ops(self, keys: List[COSEKey], op: int) -> List[COSEKey]:
-        res: List[COSEKey] = []
+    def _filter_by_key_ops(
+        self, keys: List[COSEKeyInterface], op: int
+    ) -> List[COSEKeyInterface]:
+        res: List[COSEKeyInterface] = []
         for k in keys:
             if op in k.key_ops:
                 res.append(k)
