@@ -6,12 +6,11 @@ from cbor2 import CBORTag
 
 from .cbor_processor import CBORProcessor
 from .claims import Claims
-from .claims_builder import ClaimsBuilder
 from .const import COSE_KEY_OPERATION_VALUES
 from .cose import COSE
-from .cose_key import COSEKey
+from .cose_key_interface import COSEKeyInterface
 from .exceptions import DecodeError, VerifyError
-from .recipient import Recipient
+from .recipient_interface import RecipientInterface
 
 _CWT_DEFAULT_EXPIRES_IN = 3600  # 1 hour
 _CWT_DEFAULT_LEEWAY = 60  # 1 min
@@ -40,9 +39,9 @@ class CWT(CBORProcessor):
 
         Examples:
 
-            >>> from cwt import CWT, claims, cose_key
+            >>> from cwt import CWT, COSEKey
             >>> ctx = CWT({"expires_in": 3600*24, "leeway": 10})
-            >>> key = cose_key.from_symmetric_key(alg="HS255")
+            >>> key = COSEKey.from_symmetric_key(alg="HS255")
             >>> token = ctx.encode(
             ...     {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"},
             ...     key,
@@ -50,8 +49,8 @@ class CWT(CBORProcessor):
         """
         self._expires_in = _CWT_DEFAULT_EXPIRES_IN
         self._leeway = _CWT_DEFAULT_LEEWAY
-        self._cose = COSE(options)
-        self._claims = ClaimsBuilder()
+        self._cose = COSE(kid_auto_inclusion=True, alg_auto_inclusion=True)
+        self._claim_names: Dict[str, int] = {}
         if not options:
             return
 
@@ -86,10 +85,10 @@ class CWT(CBORProcessor):
     def encode(
         self,
         claims: Union[Claims, Dict[str, Any], Dict[int, Any], bytes, str],
-        key: COSEKey,
+        key: COSEKeyInterface,
         nonce: bytes = b"",
+        recipients: Optional[List[RecipientInterface]] = None,
         tagged: bool = False,
-        recipients: Optional[List[Recipient]] = None,
     ) -> bytes:
         """
         Encodes CWT with MAC, signing or encryption.
@@ -106,8 +105,8 @@ class CWT(CBORProcessor):
         Args:
             claims (Union[Claims, Dict[str, Any], Dict[int, Any], bytes, str]): A CWT
                 claims object, or a JWT claims object, text string or byte string.
-            key (COSEKey): A COSE key used to generate a MAC for the claims.
-            recipients (List[Recipient]): A list of recipient information structures.
+            key (COSEKeyInterface): A COSE key used to generate a MAC for the claims.
+            recipients (List[RecipientInterface]): A list of recipient information structures.
             tagged (bool): An indicator whether the response is wrapped by CWT tag(61)
                 or not.
         Returns:
@@ -117,19 +116,19 @@ class CWT(CBORProcessor):
             EncodeError: Failed to encode the claims.
         """
         if isinstance(claims, Claims):
-            return self._encode(claims, key, nonce, tagged, recipients)
+            return self._encode(claims, key, nonce, recipients, tagged)
         if isinstance(claims, str):
             claims = claims.encode("utf-8")
         if isinstance(claims, bytes):
             try:
-                claims = self._claims.from_json(claims)
+                claims = Claims.from_json(claims, self._claim_names)
             except ValueError:
-                return self._encode(claims, key, nonce, tagged, recipients)
+                return self._encode(claims, key, nonce, recipients, tagged)
         else:
             # Following code causes mypy error:
             # for k, v in claims.items():
             #     if isinstance(k, str):
-            #         claims = self._claims.from_json(claims)
+            #         claims = Claims.from_json(claims)
             #     break
             # To avoid the error:
             json_claims: Dict[str, Any] = {}
@@ -137,15 +136,15 @@ class CWT(CBORProcessor):
                 if isinstance(k, str):
                     json_claims[k] = v
             if json_claims:
-                claims = self._claims.from_json(json_claims)
-        return self._encode(claims, key, nonce, tagged, recipients)
+                claims = Claims.from_json(json_claims, self._claim_names)
+        return self._encode(claims, key, nonce, recipients, tagged)
 
     def encode_and_mac(
         self,
         claims: Union[Claims, Dict[int, Any], bytes],
-        key: COSEKey,
+        key: COSEKeyInterface,
+        recipients: Optional[List[RecipientInterface]] = None,
         tagged: bool = False,
-        recipients: Optional[List[Recipient]] = None,
     ) -> bytes:
         """
         Encodes with MAC.
@@ -153,8 +152,8 @@ class CWT(CBORProcessor):
         Args:
             claims (Union[Claims, Dict[int, Any], bytes]): A CWT claims object or byte
                 string.
-            key (COSEKey): A COSE key used to generate a MAC for the claims.
-            recipients (List[Recipient]): A list of recipient information structures.
+            key (COSEKeyInterface): A COSE key used to generate a MAC for the claims.
+            recipients (List[RecipientInterface]): A list of recipient information structures.
             tagged (bool): An indicator whether the response is wrapped by CWT tag(61)
                 or not.
         Returns:
@@ -179,7 +178,7 @@ class CWT(CBORProcessor):
     def encode_and_sign(
         self,
         claims: Union[Claims, Dict[int, Any], bytes],
-        key: Union[COSEKey, List[COSEKey]],
+        key: Union[COSEKeyInterface, List[COSEKeyInterface]],
         tagged: bool = False,
     ) -> bytes:
         """
@@ -188,7 +187,7 @@ class CWT(CBORProcessor):
         Args:
             claims (Claims, Union[Dict[int, Any], bytes]): A CWT claims object or byte
                 string.
-            key (Union[COSEKey, List[COSEKey]]): A COSE key or a list of the keys used
+            key (Union[COSEKeyInterface, List[COSEKeyInterface]]): A COSE key or a list of the keys used
                 to sign claims.
             tagged (bool): An indicator whether the response is wrapped by CWT tag(61)
                 or not.
@@ -212,10 +211,10 @@ class CWT(CBORProcessor):
     def encode_and_encrypt(
         self,
         claims: Union[Claims, Dict[int, Any], bytes],
-        key: COSEKey,
+        key: COSEKeyInterface,
         nonce: bytes = b"",
+        recipients: Optional[List[RecipientInterface]] = None,
         tagged: bool = False,
-        recipients: Optional[List[Recipient]] = None,
     ) -> bytes:
         """
         Encodes CWT with encryption.
@@ -223,9 +222,9 @@ class CWT(CBORProcessor):
         Args:
             claims (Claims, Union[Dict[int, Any], bytes]): A CWT claims object or byte
                 string.
-            key (COSEKey): A COSE key used to encrypt the claims.
+            key (COSEKeyInterface): A COSE key used to encrypt the claims.
             nonce (bytes): A nonce for encryption.
-            recipients (List[Recipient]): A list of recipient information structures.
+            recipients (List[RecipientInterface]): A list of recipient information structures.
             tagged (bool): An indicator whether the response is wrapped by CWT tag(61)
                 or not.
         Returns:
@@ -258,14 +257,17 @@ class CWT(CBORProcessor):
         return self._dumps(res)
 
     def decode(
-        self, data: bytes, key: Union[COSEKey, List[COSEKey]], no_verify: bool = False
+        self,
+        data: bytes,
+        key: Union[COSEKeyInterface, List[COSEKeyInterface]],
+        no_verify: bool = False,
     ) -> Union[Dict[int, Any], bytes]:
         """
         Verifies and decodes CWT.
 
         Args:
             data (bytes): A byte string of an encoded CWT.
-            key (Union[COSEKey, List[COSEKey]]): A COSE key or a list of the keys
+            key (Union[COSEKeyInterface, List[COSEKeyInterface]]): A COSE key or a list of the keys
                 used to verify and decrypt the encoded CWT.
             no_verify (bool): An indicator whether token verification is skiped
                 or not.
@@ -279,7 +281,9 @@ class CWT(CBORProcessor):
         cwt: Union[bytes, CBORTag, Dict[int, Any]] = self._loads(data)
         if isinstance(cwt, CBORTag) and cwt.tag == CWT.CBOR_TAG:
             cwt = cwt.value
-        keys: List[COSEKey] = [key] if isinstance(key, COSEKey) else key
+        keys: List[COSEKeyInterface] = (
+            [key] if isinstance(key, COSEKeyInterface) else key
+        )
         while isinstance(cwt, CBORTag):
             cwt = self._cose.decode(cwt, keys)
             cwt = self._loads(cwt)
@@ -289,14 +293,11 @@ class CWT(CBORProcessor):
 
     def set_private_claim_names(self, claim_names: Dict[str, int]):
         """
-        Sets private claim definitions. This function call is redirected to the
-        internal :class:`ClaimsBuilder <cwt.ClaimsBuilder>`'s
-        :func:`set_private_claim_names <cwt.ClaimsBuilder.set_private_claim_names>`
-        directly. The definitions will be used in :func:`encode <cwt.CWT.encode>`
-        when it is called with JSON-based claims.
+        Sets private claim definitions. The definitions will be used in
+        :func:`encode <cwt.CWT.encode>` when it is called with JSON-based claims.
 
         Args:
-            claims (Dict[str, int]): A set of private claim definitions which
+            claim_names (Dict[str, int]): A set of private claim definitions which
                 consist of a readable claim name(str) and a claim key(int).
                 The claim key should be less than -65536 but you  can use the
                 numbers other than pre-registered numbers listed in
@@ -304,15 +305,16 @@ class CWT(CBORProcessor):
         Raises:
             ValueError: Invalid arguments.
         """
-        return self._claims.set_private_claim_names(claim_names)
+        self._claim_names = claim_names
+        return
 
     def _encode(
         self,
         claims: Union[Claims, Dict[Any, Any], bytes],
-        key: COSEKey,
+        key: COSEKeyInterface,
         nonce: bytes = b"",
+        recipients: Optional[List[RecipientInterface]] = None,
         tagged: bool = False,
-        recipients: Optional[List[Recipient]] = None,
     ) -> bytes:
         if COSE_KEY_OPERATION_VALUES["sign"] in key.key_ops:
             if [ops for ops in key.key_ops if ops in [3, 4, 9, 10]]:
@@ -321,11 +323,11 @@ class CWT(CBORProcessor):
         if COSE_KEY_OPERATION_VALUES["encrypt"] in key.key_ops:
             if [ops for ops in key.key_ops if ops in [1, 2, 9, 10]]:
                 raise ValueError("The key operation could not be specified.")
-            return self.encode_and_encrypt(claims, key, nonce, tagged, recipients)
+            return self.encode_and_encrypt(claims, key, nonce, recipients, tagged)
         if COSE_KEY_OPERATION_VALUES["MAC create"] in key.key_ops:
             if [ops for ops in key.key_ops if ops in [1, 2, 3, 4]]:
                 raise ValueError("The key operation could not be specified.")
-            return self.encode_and_mac(claims, key, tagged, recipients)
+            return self.encode_and_mac(claims, key, recipients, tagged)
         raise ValueError("The key operation could not be specified.")
 
     def _validate(self, claims: Union[Dict[int, Any], bytes]):
@@ -339,7 +341,7 @@ class CWT(CBORProcessor):
             if nested.tag not in [16, 96, 17, 97, 18, 98]:
                 raise ValueError(f"Unsupported or unknown CBOR tag({nested.tag}).")
             return
-        self._claims.validate(claims)
+        Claims.validate(claims)
         return
 
     def _verify(self, claims: Union[Dict[int, Any], bytes]):
