@@ -12,7 +12,9 @@ import cbor2
 import pytest
 from cbor2 import CBORTag
 
-from cwt import CWT, COSEKey, DecodeError, Recipient, VerifyError, claims, cose_key
+from cwt import CWT, Claims, COSEKey, DecodeError, VerifyError
+from cwt.cose_key_interface import COSEKeyInterface
+from cwt.recipient_interface import RecipientInterface
 
 from .utils import key_path, now
 
@@ -59,9 +61,9 @@ class TestCWT:
         assert "should be" in str(err.value)
 
     def test_cwt_encode_with_claims_object(self, ctx):
-        key = cose_key.from_symmetric_key(alg="HS256")
+        key = COSEKey.from_symmetric_key(alg="HS256")
         token = ctx.encode(
-            claims.from_json(
+            Claims.from_json(
                 {"iss": "https://as.example", "sub": "someone", "cti": b"123"}
             ),
             key,
@@ -74,10 +76,10 @@ class TestCWT:
     @pytest.mark.parametrize(
         "invalid_key",
         [
-            COSEKey({1: 4, 2: b"123", 3: 1, 4: []}),
-            COSEKey({1: 4, 2: b"123", 3: 1, 4: [1, 3]}),
-            COSEKey({1: 4, 2: b"123", 3: 1, 4: [3, 9]}),
-            COSEKey({1: 4, 2: b"123", 3: 1, 4: [9, 4]}),
+            COSEKeyInterface({1: 4, 2: b"123", 3: 1, 4: []}),
+            COSEKeyInterface({1: 4, 2: b"123", 3: 1, 4: [1, 3]}),
+            COSEKeyInterface({1: 4, 2: b"123", 3: 1, 4: [3, 9]}),
+            COSEKeyInterface({1: 4, 2: b"123", 3: 1, 4: [9, 4]}),
         ],
     )
     def test_cwt_encode_with_invalid_key(self, ctx, invalid_key):
@@ -87,7 +89,7 @@ class TestCWT:
         assert "The key operation could not be specified." in str(err.value)
 
     def test_cwt_encode_and_mac_with_default_alg(self, ctx):
-        key = cose_key.from_symmetric_key("mysecret")
+        key = COSEKey.from_symmetric_key("mysecret")
         token = ctx.encode_and_mac(
             {1: "https://as.example", 2: "someone", 7: b"123"}, key
         )
@@ -111,7 +113,7 @@ class TestCWT:
         ],
     )
     def test_cwt_encode_and_mac_with_valid_alg_hmac(self, ctx, alg):
-        key = cose_key.from_symmetric_key("mysecret", alg=alg)
+        key = COSEKey.from_symmetric_key("mysecret", alg=alg)
         token = ctx.encode_and_mac(
             {1: "https://as.example", 2: "someone", 7: b"123"}, key
         )
@@ -121,7 +123,7 @@ class TestCWT:
         assert 7 in decoded and decoded[7] == b"123"
 
     def test_cwt_encode_and_mac_with_tagged(self, ctx):
-        key = cose_key.from_symmetric_key("mysecret", alg="HMAC 256/64")
+        key = COSEKey.from_symmetric_key("mysecret", alg="HMAC 256/64")
         token = ctx.encode_and_mac(
             {1: "https://as.example", 2: "someone", 7: b"123"},
             key,
@@ -133,8 +135,8 @@ class TestCWT:
         assert 7 in decoded and decoded[7] == b"123"
 
     def test_cwt_encode_and_mac_with_recipient(self, ctx):
-        recipient = Recipient(unprotected={1: -6, 4: b"our-secret"})
-        key = cose_key.from_symmetric_key(
+        recipient = RecipientInterface(unprotected={1: -6, 4: b"our-secret"})
+        key = COSEKey.from_symmetric_key(
             "mysecret", alg="HMAC 256/64", kid="our-secret"
         )
         token = ctx.encode_and_mac(
@@ -149,7 +151,7 @@ class TestCWT:
         assert 7 in decoded and decoded[7] == b"123"
 
     def test_cwt_encode_and_mac_with_empty_recipient(self, ctx):
-        key = cose_key.from_symmetric_key("mysecret", alg="HMAC 256/64")
+        key = COSEKey.from_symmetric_key("mysecret", alg="HMAC 256/64")
         token = ctx.encode_and_mac(
             {1: "https://as.example", 2: "someone", 7: b"123"},
             key,
@@ -181,7 +183,7 @@ class TestCWT:
         ],
     )
     def test_cwt_encode_and_mac_with_invalid_args(self, ctx, invalid):
-        key = cose_key.from_symmetric_key("mysecret")
+        key = COSEKey.from_symmetric_key("mysecret")
         with pytest.raises(ValueError) as err:
             ctx.encode_and_mac(invalid, key)
             pytest.fail("encode_and_mac should fail.")
@@ -201,14 +203,14 @@ class TestCWT:
         ],
     )
     def test_cwt_encode_and_mac_with_invalid_claims(self, ctx, invalid, msg):
-        key = cose_key.from_symmetric_key("mysecret")
+        key = COSEKey.from_symmetric_key("mysecret")
         with pytest.raises(ValueError) as err:
             ctx.encode_and_mac(invalid, key)
             pytest.fail("encode_and_mac should fail.")
         assert msg in str(err.value)
 
     def test_cwt_encode_and_mac_with_invalid_cbor_format(self, ctx):
-        key = cose_key.from_symmetric_key("mysecret", alg="HMAC 256/64")
+        key = COSEKey.from_symmetric_key("mysecret", alg="HMAC 256/64")
         with pytest.raises(ValueError) as err:
             ctx.encode_and_mac(
                 b'{1: "https://as.example", 2: "someone", 7: "123"',
@@ -218,7 +220,7 @@ class TestCWT:
         assert "Invalid claim format." in str(err.value)
 
     def test_cwt_encode_and_mac_with_untagged_cbor_bytes(self, ctx):
-        key = cose_key.from_symmetric_key("mysecret", alg="HMAC 256/64")
+        key = COSEKey.from_symmetric_key("mysecret", alg="HMAC 256/64")
         with pytest.raises(ValueError) as err:
             ctx.encode_and_mac(
                 cbor2.dumps({1: "https://as.example", 2: "someone", 7: "123"}),
@@ -228,7 +230,7 @@ class TestCWT:
         assert "A bytes-formatted claims needs CBOR(COSE) Tag." in str(err.value)
 
     def test_cwt_encode_and_mac_with_invalid_tagged_cwt(self, ctx):
-        key = cose_key.from_symmetric_key("mysecret", alg="HMAC 256/64")
+        key = COSEKey.from_symmetric_key("mysecret", alg="HMAC 256/64")
         token = ctx.encode_and_mac(
             {1: "https://as.example", 2: "someone", 7: b"123"},
             key,
@@ -253,7 +255,7 @@ class TestCWT:
         ],
     )
     def test_cwt_encode_and_encrypt_with_valid_alg_aes_ccm(self, ctx, alg, nonce, key):
-        enc_key = cose_key.from_symmetric_key(key, alg=alg)
+        enc_key = COSEKey.from_symmetric_key(key, alg=alg)
         token = ctx.encode_and_encrypt(
             {1: "https://as.example", 2: "someone", 7: b"123"},
             enc_key,
@@ -265,7 +267,7 @@ class TestCWT:
         assert 7 in decoded and decoded[7] == b"123"
 
     def test_cwt_encode_and_encrypt_with_invalid_key_and_without_nonce(self, ctx):
-        enc_key = cose_key.from_symmetric_key(alg="HMAC 256/64")
+        enc_key = COSEKey.from_symmetric_key(alg="HMAC 256/64")
         with pytest.raises(ValueError) as err:
             ctx.encode_and_encrypt(
                 {1: "https://as.example", 2: "someone", 7: b"123"},
@@ -294,7 +296,7 @@ class TestCWT:
         ],
     )
     def test_cwt_encode_and_encrypt_without_key_and_nonce(self, ctx, alg):
-        enc_key = cose_key.from_symmetric_key(alg=alg)
+        enc_key = COSEKey.from_symmetric_key(alg=alg)
         token = ctx.encode_and_encrypt(
             {1: "https://as.example", 2: "someone", 7: b"123"},
             enc_key,
@@ -313,7 +315,7 @@ class TestCWT:
         ],
     )
     def test_cwt_encode_and_encrypt_with_valid_alg_aes_gcm(self, ctx, alg, key):
-        enc_key = cose_key.from_symmetric_key(key, alg=alg)
+        enc_key = COSEKey.from_symmetric_key(key, alg=alg)
         token = ctx.encode_and_encrypt(
             {1: "https://as.example", 2: "someone", 7: b"123"},
             enc_key,
@@ -327,7 +329,7 @@ class TestCWT:
     def test_cwt_encode_and_encrypt_with_tagged(self, ctx):
         key = token_bytes(16)
         nonce = token_bytes(13)
-        enc_key = cose_key.from_symmetric_key(key, alg="AES-CCM-16-64-128")
+        enc_key = COSEKey.from_symmetric_key(key, alg="AES-CCM-16-64-128")
         token = ctx.encode_and_encrypt(
             {1: "https://as.example", 2: "someone", 7: b"123"},
             enc_key,
@@ -340,10 +342,10 @@ class TestCWT:
         assert 7 in decoded and decoded[7] == b"123"
 
     def test_cwt_encode_and_encrypt_with_recipient_direct(self, ctx):
-        enc_key = cose_key.from_symmetric_key(
+        enc_key = COSEKey.from_symmetric_key(
             token_bytes(16), alg="AES-CCM-16-64-128", kid="our-secret"
         )
-        recipient = Recipient(unprotected={1: -6, 4: b"our-secret"})
+        recipient = RecipientInterface(unprotected={1: -6, 4: b"our-secret"})
         token = ctx.encode_and_encrypt(
             {1: "https://as.example", 2: "someone", 7: b"123"},
             enc_key,
@@ -370,9 +372,9 @@ class TestCWT:
         self, ctx, private_key_path, public_key_path
     ):
         with open(key_path(private_key_path)) as key_file:
-            private_key = cose_key.from_pem(key_file.read())
+            private_key = COSEKey.from_pem(key_file.read())
         with open(key_path(public_key_path)) as key_file:
-            public_key = cose_key.from_pem(key_file.read())
+            public_key = COSEKey.from_pem(key_file.read())
         token = ctx.encode_and_sign(
             {1: "https://as.example", 2: "someone", 7: b"123"},
             private_key,
@@ -401,9 +403,9 @@ class TestCWT:
     )
     def test_cwt_encode_and_sign_with_valid_alg_rsa(self, ctx, alg):
         with open(key_path("public_key_rsa.pem")) as key_file:
-            public_key = cose_key.from_pem(key_file.read(), alg=alg)
+            public_key = COSEKey.from_pem(key_file.read(), alg=alg)
         with open(key_path("private_key_rsa.pem")) as key_file:
-            private_key = cose_key.from_pem(key_file.read(), alg=alg)
+            private_key = COSEKey.from_pem(key_file.read(), alg=alg)
         token = ctx.encode_and_sign(
             {1: "https://as.example", 2: "someone", 7: b"123"},
             private_key,
@@ -415,9 +417,9 @@ class TestCWT:
 
     def test_cwt_encode_and_sign_with_tagged(self, ctx):
         with open(key_path("private_key_es256.pem")) as key_file:
-            private_key = cose_key.from_pem(key_file.read())
+            private_key = COSEKey.from_pem(key_file.read())
         with open(key_path("public_key_es256.pem")) as key_file:
-            public_key = cose_key.from_pem(key_file.read())
+            public_key = COSEKey.from_pem(key_file.read())
         token = ctx.encode_and_sign(
             {1: "https://as.example", 2: "someone", 7: b"123"},
             private_key,
@@ -430,13 +432,13 @@ class TestCWT:
 
     def test_cwt_encode_and_sign_with_multiple_signatures(self, ctx):
         with open(key_path("private_key_es256.pem")) as key_file:
-            private_key_1 = cose_key.from_pem(key_file.read(), kid="1")
+            private_key_1 = COSEKey.from_pem(key_file.read(), kid="1")
         with open(key_path("public_key_es256.pem")) as key_file:
-            public_key_1 = cose_key.from_pem(key_file.read(), kid="1")
+            public_key_1 = COSEKey.from_pem(key_file.read(), kid="1")
         with open(key_path("private_key_ed25519.pem")) as key_file:
-            private_key_2 = cose_key.from_pem(key_file.read(), kid="2")
+            private_key_2 = COSEKey.from_pem(key_file.read(), kid="2")
         with open(key_path("public_key_ed25519.pem")) as key_file:
-            public_key_2 = cose_key.from_pem(key_file.read(), kid="2")
+            public_key_2 = COSEKey.from_pem(key_file.read(), kid="2")
         token = ctx.encode_and_sign(
             {1: "https://as.example", 2: "someone", 7: b"123"},
             [private_key_1, private_key_2],
@@ -449,7 +451,7 @@ class TestCWT:
         assert 1 in decoded and decoded[1] == "https://as.example"
 
     def test_cwt_encode_and_encrypt_with_invalid_nonce(self, ctx):
-        enc_key = cose_key.from_symmetric_key(token_bytes(16), alg="AES-CCM-16-64-128")
+        enc_key = COSEKey.from_symmetric_key(token_bytes(16), alg="AES-CCM-16-64-128")
         with pytest.raises(ValueError) as err:
             ctx.encode_and_encrypt(
                 {1: "https://as.example", 2: "someone", 7: b"123"},
@@ -461,13 +463,13 @@ class TestCWT:
 
     def test_cwt_encode_and_sign_with_signatures_kid_mismatch(self, ctx):
         with open(key_path("private_key_es256.pem")) as key_file:
-            private_key_1 = cose_key.from_pem(key_file.read(), kid="1")
+            private_key_1 = COSEKey.from_pem(key_file.read(), kid="1")
         with open(key_path("public_key_es256.pem")) as key_file:
-            public_key_1 = cose_key.from_pem(key_file.read(), kid="3")
+            public_key_1 = COSEKey.from_pem(key_file.read(), kid="3")
         with open(key_path("private_key_ed25519.pem")) as key_file:
-            private_key_2 = cose_key.from_pem(key_file.read(), kid="2")
+            private_key_2 = COSEKey.from_pem(key_file.read(), kid="2")
         with open(key_path("public_key_ed25519.pem")) as key_file:
-            cose_key.from_pem(key_file.read(), kid="2")
+            COSEKey.from_pem(key_file.read(), kid="2")
         token = ctx.encode_and_sign(
             {1: "https://as.example", 2: "someone", 7: b"123"},
             [private_key_1, private_key_2],
@@ -478,11 +480,11 @@ class TestCWT:
         assert "Verification key not found." in str(err.value)
 
     def test_cwt_decode_with_invalid_mac_key(self, ctx):
-        key = cose_key.from_symmetric_key("mysecret")
+        key = COSEKey.from_symmetric_key("mysecret")
         token = ctx.encode_and_mac(
             {1: "https://as.example", 2: "someone", 7: b"123"}, key
         )
-        wrong_key = cose_key.from_symmetric_key("xxxxxxxxxx")
+        wrong_key = COSEKey.from_symmetric_key("xxxxxxxxxx")
         with pytest.raises(VerifyError) as err:
             res = ctx.decode(token, wrong_key)
             pytest.fail("decode should fail: res=%s" % vars(res))
@@ -497,17 +499,15 @@ class TestCWT:
         ],
     )
     def test_cwt_decode_with_invalid_cwt(self, ctx, invalid):
-        key = cose_key.from_symmetric_key(alg="AES-CCM-16-64-128")
+        key = COSEKey.from_symmetric_key(alg="AES-CCM-16-64-128")
         with pytest.raises(DecodeError) as err:
             ctx.decode(invalid, key)
             pytest.fail("decode should fail.")
         assert "Failed to decode." in str(err.value)
 
     def test_cwt_decode_with_invalid_enc_key(self, ctx):
-        enc_key = cose_key.from_symmetric_key(token_bytes(16), alg="AES-CCM-16-64-128")
-        wrong_key = cose_key.from_symmetric_key(
-            token_bytes(16), alg="AES-CCM-16-64-128"
-        )
+        enc_key = COSEKey.from_symmetric_key(token_bytes(16), alg="AES-CCM-16-64-128")
+        wrong_key = COSEKey.from_symmetric_key(token_bytes(16), alg="AES-CCM-16-64-128")
         token = ctx.encode_and_encrypt(
             {1: "https://as.example", 2: "someone", 7: b"123"},
             enc_key,
@@ -538,7 +538,7 @@ class TestCWT:
         ],
     )
     def test_cwt_decode_with_invalid_claim(self, ctx, invalid, msg):
-        mac_key = cose_key.from_symmetric_key("mysecret")
+        mac_key = COSEKey.from_symmetric_key("mysecret")
         token = ctx.encode_and_mac(invalid, mac_key)
         with pytest.raises(VerifyError) as err:
             ctx.decode(token, mac_key)
@@ -546,7 +546,7 @@ class TestCWT:
         assert msg in str(err.value)
 
     def test_cwt_decode_with_invalid_tagged_cwt(self, ctx):
-        key = cose_key.from_symmetric_key("mysecret", alg="HMAC 256/64")
+        key = COSEKey.from_symmetric_key("mysecret", alg="HMAC 256/64")
         token = ctx.encode_and_mac(
             {1: "https://as.example", 2: "someone", 7: b"123"},
             key,
